/** @file
Provides services to print debug messages via system trace.
 
Copyright (c) 2014 - 2015, ASSET InterTech Inc. All rights reserved.

This software and associated documentation (if any) is furnished under
a license and may only be used or copied in accordance with the terms of
the license. Except as permitted by such license, no part of this software
or documentation may be reproduced, stored in a retrieval system, or
transmitted in any form or by any means without the express written consent
of ASSET InterTech Inc.

**/

#ifndef __TRACE_DEMO_H__
#define __TRACE_DEMO_H__

#define MAX_ENTRIES 100

// CMOS access Port address

#define PORT_70  0x70
#define PORT_71  0x71
#define PORT_72  0x72
#define PORT_73  0x73

#define CMOS_LOW_MEM_ST   0
#define CMOS_LOW_MEM_END  127

#define CMOS_HIGH_MEM_ST  128
#define CMOS_HIGH_MEM_END 255

#define CMOS_START_ADDR   40
#define CMOS_END_ADDR     255

//
// Don't set this value more than 254 and equal to 127
//
#define CMOS_CHECKSUM_ADDR_LOW  254  
#define CMOS_CHECKSUM_ADDR_HIGH (CMOS_CHECKSUM_ADDR_LOW + 1)

typedef union {
  struct {
    UINT8 sqrt:8;
    UINT8 fact:8;
	UINT8 sqrt2:8;
	UINT8 reserved;
  } Bits;
  UINT32 Data;
} CMOS;

struct sysHost {
	UINT16 nearsqrt[MAX_ENTRIES];
	UINT16 nearsqrt2[MAX_ENTRIES];
	UINT64 factorial[MAX_ENTRIES];
	UINT8 fact;
	UINT8 sqrt;
	UINT8 sqrt2;
	UINT8 cmos_block[256];
	CMOS cmos_data;
};

/**
  Returns the nearest integer from the square root of value.
  
  @param[in]  Value			 Value to calculate the Square Root of

  @retval     Contains the nearest Integer Square of "value"
**/
UINT8
calcNearSqrt (
  IN UINT8 value
  );

UINT32
ReadCmosLocal (
  IN UINT8 CmosAddress,
  OUT UINT8 *Data
  );

/**
  Writes an 8-bit to I/O port.

  Writes the 8-bit to I/O port specified by PortAdd with the value specified by Value
  and returns Value.

  If 8-bit I/O port operations are not supported, then ASSERT().

  @param[in]  PortAdd  The I/O port to write.
  @param[in]  Value    The value to write to the I/O port.

  @retval     Value    The value written to the I/O port.
**/
UINT8
WriteIoPortTrace (
  IN UINT8 PortAdd,
  IN UINT8 Value
  );


/**
  Reads an 8-bit from I/O port.

  Reads the 8-bit from I/O port specified by PortAdd and returns value.

  If 8-bit I/O port operations are not supported, then ASSERT().

  @param[in]  PortAdd  The I/O port to read.

  @retval     Value    The value read from the I/O port.
**/
UINT8
ReadIoPortTrace (
  IN UINT8 PortAdd
  );


/**
  Read a byte value from a CMOS address.

  @param [in] Address   Location to read from CMOS

  @return The byte value read from the CMOS address.
**/
UINT8
EFIAPI
ReadCmos8 (
  IN  UINT8 Address
  );

#endif