/** @file
Provides services to print debug messages via system trace.
 
Copyright (c) 2014 - 2021, ASSET InterTech Inc. All rights reserved.

This program and the accompanying materials
are licensed and made available under the terms and conditions of the BSD License
which accompanies this distribution.  The full text of the license may be found at
http://opensource.org/licenses/bsd-license.php

THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.

**/

#ifndef __DEBUG_LIB_SYSTEM_TRACE_H__
#define __DEBUG_LIB_SYSTEM_TRACE_H__

// STM packet generation offsets - Invariant Timing
#define I_DMTS 		0x18
#define I_DM 		0x08
#define I_DTS 		0x10
#define I_D 		0x00
#define I_FLAGTS 	0x34
#define I_FLAG  	0x30

// STM write memory area
#define STM_CHANNEL_TOTAL	0x80
#define STM_CHANNEL_SIZE    0x40											// 64 write locations
#define STM_SWMSTR			0x10
#define STM_WRITE_REGION	pPCH_TRACE_HUB_FW_BASE_ADDRESS
#define STM_MASTER_SIZE		(STM_CHANNEL_TOTAL * STM_CHANNEL_SIZE)
#define nMaster1            0x74
#define nMaster2            0x71
#define nMaster3            0x75
#define nMaster4            0x71
#define nMaster5            0x71
#define nMaster6            0x77
#define nChannel1           0x43
#define nChannel2           0x25
#define nChannel3           0x20
#define nChannel4           0x26
#define nChannel5           0x27
#define nChannel6           0x20

// Parameters for rcHprintf master and channel
#define rcHPRINTF_MASTER			nMaster3
#define rcHPRINTF_CHANNEL			nChannel3

#define RC_MSG_LEVEL_CHK_DISABLE	1
#define DEBUG_LIB_SYSTEM_TRACE		1

//
// Hosted printf macros for rcprintf replacement (0, 1, 2, 3 and 4 argument counts)
//
#ifndef DEBUG_LIB_SYSTEM_TRACE
	#define MrcHprintf0(MrcDebugMsgLevel, MsgLevel, format) MrcPrintf (MrcDebugMsgLevel, MsgLevel, format)
	#define MrcHprintf1(MrcDebugMsgLevel, MsgLevel, format, arg1) MrcPrintf (MrcDebugMsgLevel, MsgLevel, format)
	#define MrcHprintf2(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2) MrcPrintf (MrcDebugMsgLevel, MsgLevel, format)
	#define MrcHprintf3(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2, arg3) MrcPrintf (MrcDebugMsgLevel, MsgLevel, format)
	#define MrcHprintf4(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2, arg3, arg4) MrcPrintf (MrcDebugMsgLevel, MsgLevel, format)

	#define DEBUG_ASSET(MsgLevel, format) DEBUG (MsgLevel, format)
	#define DEBUG_ASSET1(MsgLevel, format, arg1) DEBUG (MsgLevel, format)
	
#else
  #ifndef RC_MSG_LEVEL_CHK_DISABLE
	#define MrcHprintf0(MrcDebugMsgLevel, MsgLevel, format) { \
	  if ( (MrcDebugMsgLevel & MsgLevel) != 0) { \
		hPrintf0(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format); \
	  } \
	}
	#define MrcHprintf1(MrcDebugMsgLevel, MsgLevel, format, arg1) { \
	  if ( (MrcDebugMsgLevel & MsgLevel) != 0) { \
		hPrintf1(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1); \
	  } \
	}

	#define MrcHprintf2(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2) { \
	  if ( (MrcDebugMsgLevel & MsgLevel) != 0) { \
		hPrintf2(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1, arg2); \
	  } \
	}
	#define MrcHprintf3(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2, arg3) { \
	  if ( (MrcDebugMsgLevel & MsgLevel) != 0) { \
		hPrintf3(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1, arg2, arg3); \
	  } \
	}
	#define MrcHprintf4(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2, arg3, arg4) { \
	  if ( (MrcDebugMsgLevel & MsgLevel) != 0) { \
		hPrintf4(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1, arg2, arg3, arg4); \
	  } \
	}
	
	#define DEBUG_ASSET(MsgLevel, format) { \
	  if ( MsgLevel != 0 ) { \
		hPrintf0(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format); \
	  } \
	}
	#define DEBUG_ASSET1(MsgLevel, format, arg1) { \
	  if ( MsgLevel != 0 ) { \
		hPrintf1(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1); \
	  } \
	}
	
  #else
	#define MrcHprintf0(MrcDebugMsgLevel, MsgLevel, format) hPrintf0(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format)
	#define MrcHprintf1(MrcDebugMsgLevel, MsgLevel, format, arg1) hPrintf1(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1)
	#define MrcHprintf2(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2) hPrintf2(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1, arg2)
	#define MrcHprintf3(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2, arg3) hPrintf3(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1, arg2, arg3)
	#define MrcHprintf4(MrcDebugMsgLevel, MsgLevel, format, arg1, arg2, arg3, arg4) hPrintf4(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1, arg2, arg3, arg4)

	#define DEBUG_ASSET(MsgLevel, format) hPrintf0(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format)
	#define DEBUG_ASSET1(MsgLevel, format, arg1) hPrintf1(rcHPRINTF_MASTER, rcHPRINTF_CHANNEL, format, arg1)
  #endif

  #define hPrintf0(master, channel, format) { \
	volatile UINT32 addr = (UINT32)(STM_WRITE_REGION + ((master - STM_SWMSTR) * STM_MASTER_SIZE) + (channel * STM_CHANNEL_SIZE)); \
	*(UINT8 *)(addr + I_DMTS) = (UINT8)sizeof(format); \
	*(UINT32 *)(addr + I_D) = (UINT32)&format; \
  }

  #define hPrintf1(master, channel, format, arg1) { \
	volatile UINT32 addr = (UINT32)(STM_WRITE_REGION + ((master - STM_SWMSTR) * STM_MASTER_SIZE) + (channel * STM_CHANNEL_SIZE));; \
	*(UINT8 *)(addr + I_DMTS) = (UINT8)sizeof(format); \
	*(UINT32 *)(addr + I_D) = (UINT32)&format; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg1; \
  }

  #define hPrintf2(master, channel, format, arg1, arg2) { \
	volatile UINT32 addr = (UINT32)(STM_WRITE_REGION + ((master - STM_SWMSTR) * STM_MASTER_SIZE) + (channel * STM_CHANNEL_SIZE));; \
	*(UINT8 *)(addr + I_DMTS) = (UINT8)sizeof(format); \
	*(UINT32 *)(addr + I_D) = (UINT32)&format; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg1; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg2; \
  }

  #define hPrintf3(master, channel, format, arg1, arg2, arg3) { \
	volatile UINT32 addr = (UINT32)(STM_WRITE_REGION + ((master - STM_SWMSTR) * STM_MASTER_SIZE) + (channel * STM_CHANNEL_SIZE)); \
	*(UINT8 *)(addr + I_DMTS) = (UINT8)sizeof(format); \
	*(UINT32 *)(addr + I_D) = (UINT32)&format; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg1; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg2; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg3; \
  }

  #define hPrintf4(master, channel, format, arg1, arg2, arg3, arg4) { \
	volatile UINT32 addr = (UINT32)(STM_WRITE_REGION + ((master - STM_SWMSTR) * STM_MASTER_SIZE) + (channel * STM_CHANNEL_SIZE)); \
	*(UINT8 *)(addr + I_DMTS) = (UINT8)sizeof(format); \
	*(UINT32 *)(addr + I_D) = (UINT32)&format; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg1; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg2; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg3; \
	*(UINT64 *)(addr + I_D) = (UINT64)arg4; \
  }
#endif

#define STM_WRITE_REGION_GPRS (STM_WRITE_REGION + (((rcHPRINTF_MASTER + 1) - STM_SWMSTR) * STM_MASTER_SIZE) + ((rcHPRINTF_CHANNEL) * STM_CHANNEL_SIZE))
#define STM_WRITE_REGION_GPR_A (STM_WRITE_REGION + (((rcHPRINTF_MASTER + 1) - STM_SWMSTR) * STM_MASTER_SIZE) + ((rcHPRINTF_CHANNEL + 1) * STM_CHANNEL_SIZE) + I_DMTS)
#define STM_WRITE_REGION_GPR_B (STM_WRITE_REGION + (((rcHPRINTF_MASTER + 1) - STM_SWMSTR) * STM_MASTER_SIZE) + ((rcHPRINTF_CHANNEL + 2) * STM_CHANNEL_SIZE) + I_DMTS)
#define STM_WRITE_REGION_GPR_C (STM_WRITE_REGION + (((rcHPRINTF_MASTER + 1) - STM_SWMSTR) * STM_MASTER_SIZE) + ((rcHPRINTF_CHANNEL + 3) * STM_CHANNEL_SIZE) + I_DMTS)
#define STM_WRITE_REGION_GPR_D (STM_WRITE_REGION + (((rcHPRINTF_MASTER + 1) - STM_SWMSTR) * STM_MASTER_SIZE) + ((rcHPRINTF_CHANNEL + 4) * STM_CHANNEL_SIZE) + I_DMTS)
#define STM_WRITE_REGION_RIP (STM_WRITE_REGION + (((rcHPRINTF_MASTER + 1) - STM_SWMSTR) * STM_MASTER_SIZE) + ((rcHPRINTF_CHANNEL + 5) * STM_CHANNEL_SIZE))

#define DEBUG_GPR_A() { \
	volatile UINT32 addrA = (UINT32)(STM_WRITE_REGION_GPR_A); \
	*(UINT64 *)(addrA) = (UINT64)AsmReadGprA(); \
}
#define DEBUG_GPR_B() { \
	volatile UINT32 addrB = (UINT32)(STM_WRITE_REGION_GPR_B); \
	*(UINT64 *)(addrB) = (UINT64)AsmReadGprB(); \
}
#define DEBUG_GPR_C() { \
	volatile UINT32 addrC = (UINT32)(STM_WRITE_REGION_GPR_C); \
	*(UINT64 *)(addrC) = (UINT64)AsmReadGprC(); \
}
#define DEBUG_GPR_D() { \
	volatile UINT32 addrD = (UINT32)(STM_WRITE_REGION_GPR_D); \
	*(UINT64 *)(addrD) = (UINT64)AsmReadGprD(); \
}
#define DEBUG_RIP() { \
	volatile UINT32 addrR = (UINT32)(STM_WRITE_REGION_RIP); \
	*(UINT64 *)(addrR + I_DTS) = (UINT64)AsmReadRip(); \
	*(UINT32 *)(addrR + I_FLAGTS) = (UINT8)(0); \
}
#define DEBUG_GPRS() { \
	volatile UINT32 addrG = (UINT32)(STM_WRITE_REGION_GPRS); \
	*(UINT64 *)(addrG + I_DTS) = (UINT64)AsmReadGprA(); \
	*(UINT64 *)(addrG + I_DTS) = (UINT64)AsmReadGprB(); \
	*(UINT64 *)(addrG + I_DTS) = (UINT64)AsmReadGprC(); \
	*(UINT64 *)(addrG + I_DTS) = (UINT64)AsmReadGprD(); \
	*(UINT32 *)(addrG + I_FLAGTS) = (UINT8)(0); \
}

/**
  Write data from buffer to system trace device. 
 
  Writes NumberOfBytes data bytes from Buffer to the system trace device.  
  If Buffer is NULL, then ASSERT(). 
  If NumberOfBytes is zero, then return 0.

  @param  Buffer           Pointer to the data buffer to be written.
  @param  NumberOfBytes    Number of bytes to written to the system trace device.

**/
UINTN
EFIAPI
SystemTraceWrite (
  IN UINT8     *Buffer,
  IN UINTN     NumberOfBytes,
  IN VA_LIST   Marker
);

UINT8 getArgCnt(const char *strMsg);

#endif
